@@ -1,10 +1,13 @@
-use std::fmt::Debug;
-use std::ops::{Deref, DerefMut};
-use std::sync::{Arc, RwLock};
-use yew::prelude::*;
-
 use super::TableRenderer;
+use std::{
+    fmt::Debug,
+    sync::{
+        atomic::{AtomicUsize, Ordering},
+        Arc, RwLock,
+    },
+};
 
+/// A model providing data for a table.
 pub trait TableModel: Debug + Default + PartialEq + Clone {
     type Item: TableRenderer;
 
@@ -20,7 +23,7 @@ pub trait TableModel: Debug + Default + PartialEq + Clone {
     fn is_expanded(&self, index: usize) -> bool;
 
     /// Set the expanded state of the entry
-    fn set_expanded(&mut self, index: usize, state: bool) -> ShouldRender;
+    fn set_expanded(&self, index: usize, state: bool) -> bool;
 
     fn map<F, R>(&self, f: F) -> Vec<R>
     where
@@ -44,134 +47,15 @@ impl<T> TableModelEntry<T> {
     }
 }
 
-#[derive(Clone, Debug, PartialEq)]
-pub struct SimpleTableModel<T>
-where
-    T: TableRenderer + Clone + Debug + PartialEq,
-{
-    entries: Vec<TableModelEntry<T>>,
-}
-
-impl<T> Deref for SimpleTableModel<T>
-where
-    T: TableRenderer + Clone + Debug + PartialEq,
-{
-    type Target = Vec<TableModelEntry<T>>;
-
-    fn deref(&self) -> &Self::Target {
-        &self.entries
-    }
-}
-
-impl<T> DerefMut for SimpleTableModel<T>
-where
-    T: TableRenderer + Clone + Debug + PartialEq,
-{
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.entries
-    }
-}
-
-impl<T> Default for SimpleTableModel<T>
-where
-    T: TableRenderer + Clone + Debug + PartialEq,
-{
-    fn default() -> Self {
-        Self { entries: vec![] }
-    }
-}
-
-impl<T> From<Vec<T>> for SimpleTableModel<T>
-where
-    T: TableRenderer + Clone + Debug + PartialEq,
-{
-    fn from(entries: Vec<T>) -> Self {
-        let mut result = Vec::with_capacity(entries.len());
-
-        let mut index = 0;
-        for e in entries {
-            result.push(TableModelEntry {
-                value: e,
-                index,
-                expanded: false,
-            });
-            index += 1;
-        }
-
-        Self { entries: result }
-    }
-}
-
-impl<T> TableModel for SimpleTableModel<T>
-where
-    T: TableRenderer + Clone + Debug + PartialEq + 'static,
-{
-    type Item = T;
-
-    fn len(&self) -> usize {
-        self.entries.len()
-    }
-
-    fn is_expanded(&self, index: usize) -> bool {
-        self.entries.is_expanded(index)
-    }
-
-    fn set_expanded(&mut self, index: usize, state: bool) -> bool {
-        self.entries.set_expanded(index, state)
-    }
-
-    fn map<F, R>(&self, f: F) -> Vec<R>
-    where
-        F: Fn(&TableModelEntry<T>) -> R,
-    {
-        self.entries.map(f)
-    }
-}
-
-impl<T> TableModel for Vec<TableModelEntry<T>>
-where
-    T: TableRenderer + Clone + Debug + PartialEq + 'static,
-{
-    type Item = T;
-
-    fn len(&self) -> usize {
-        self.len()
-    }
-
-    fn is_expanded(&self, index: usize) -> bool {
-        self.get(index).map(|e| e.expanded).unwrap_or(false)
-    }
-
-    fn set_expanded(&mut self, index: usize, state: bool) -> bool {
-        if let Some(entry) = self.get_mut(index) {
-            if entry.expanded != state {
-                entry.expanded = state;
-                true
-            } else {
-                false
-            }
-        } else {
-            false
-        }
-    }
-
-    fn map<F, R>(&self, f: F) -> Vec<R>
-    where
-        F: Fn(&TableModelEntry<T>) -> R,
-    {
-        let mut result = Vec::new();
-        for entry in self {
-            result.push(f(entry));
-        }
-        result
-    }
-}
-
 #[derive(Clone, Debug)]
 pub struct SharedTableModel<T> {
     entries: Arc<RwLock<Vec<TableModelEntry<T>>>>,
+    generation: usize,
+    id: usize,
 }
 
+static ID: AtomicUsize = AtomicUsize::new(0);
+
 impl<T> From<Vec<T>> for SharedTableModel<T> {
     fn from(initial_entries: Vec<T>) -> Self {
         let mut entries = Vec::with_capacity(initial_entries.len());
@@ -182,13 +66,21 @@ impl<T> From<Vec<T>> for SharedTableModel<T> {
             index += 1;
         }
 
+        let id = ID.fetch_add(1, Ordering::SeqCst);
+
         Self {
             entries: Arc::new(RwLock::new(entries)),
+            generation: 0,
+            id,
         }
     }
 }
 
 impl<T> SharedTableModel<T> {
+    pub fn new() -> Self {
+        vec![].into()
+    }
+
     fn new_entry(entry: T, index: usize) -> TableModelEntry<T> {
         TableModelEntry {
             expanded: false,
@@ -199,15 +91,18 @@ impl<T> SharedTableModel<T> {
 
     pub fn push(&mut self, entry: T) {
         let mut entries = self.entries.write().unwrap();
+        self.generation += 1;
         let index = entries.len();
         entries.push(Self::new_entry(entry, index))
     }
 
     pub fn insert(&mut self, index: usize, entry: T) {
         let mut entries = self.entries.write().unwrap();
+        self.generation += 1;
+
         entries.insert(index, Self::new_entry(entry, index));
 
-        // now we need to re-index everything after index
+        // now we need to re-index everything after insert
 
         for entry in &mut entries[index + 1..] {
             entry.index += 1;
@@ -216,11 +111,14 @@ impl<T> SharedTableModel<T> {
 
     pub fn pop(&mut self) -> Option<TableModelEntry<T>> {
         let mut entries = self.entries.write().unwrap();
+        self.generation += 1;
         entries.pop()
     }
 
     pub fn clear(&mut self) {
-        self.entries.write().unwrap().clear();
+        let entries = self.entries.write();
+        self.generation += 1;
+        entries.unwrap().clear();
     }
 }
 
@@ -230,22 +128,16 @@ impl<T> Default for SharedTableModel<T> {
     }
 }
 
+/// Shared models are equal by their instance id and generation. NOT by their content.
+///
+/// This is required, because components need to refresh when their view on the model has changed,
+/// as the content only exists once in memory.
 impl<T> PartialEq for SharedTableModel<T>
 where
     T: Clone + PartialEq,
 {
     fn eq(&self, other: &Self) -> bool {
-        let others: Vec<_> = {
-            other
-                .entries
-                .read()
-                .map(|other| other.clone())
-                .unwrap_or(vec![])
-        };
-        self.entries
-            .read()
-            .map(|entries| entries.deref().eq(&others))
-            .unwrap_or(false)
+        self.generation == other.generation && self.id == other.id
     }
 }
 
@@ -262,11 +154,12 @@ where
     fn is_expanded(&self, index: usize) -> bool {
         self.entries
             .read()
-            .map(|e| e.is_expanded(index))
+            .ok()
+            .and_then(|e| e.get(index).map(|e| e.expanded))
             .unwrap_or(false)
     }
 
-    fn set_expanded(&mut self, index: usize, state: bool) -> bool {
+    fn set_expanded(&self, index: usize, state: bool) -> bool {
         let mut entries = self.entries.write().unwrap();
 
         if let Some(entry) = entries.get_mut(index) {
@@ -297,3 +190,32 @@ where
         result
     }
 }
+
+#[cfg(test)]
+mod test {
+    use crate::SharedTableModel;
+
+    #[test]
+    fn test() {
+        // create two linked models
+        let mut m1 = SharedTableModel::<String>::new();
+        let m2 = m1.clone();
+
+        // push data
+        m1.push("Foo".into());
+
+        // the models must not be equal, as one of them was modified
+        assert_ne!(m1, m2);
+
+        // when we clone it again, they must be equal
+        let m3 = m1.clone();
+        assert_eq!(m1, m3);
+    }
+
+    #[test]
+    fn test_different_models() {
+        let m1 = SharedTableModel::<String>::new();
+        let m2 = SharedTableModel::<String>::new();
+        assert_ne!(m1, m2);
+    }
+}
